# Elastic tabstops converter
# convert between using spaces to align columns and elastic tabstops
# see: https://nickgravgaard.com/elastic-tabstops/
#
# This version released: 2010-04-20 (4th release)

# Copyright (C) 2007-2010 Nick Gravgaard
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


# This file tries to follow the Style Guide for Python Code (PEP 8) *EXCEPT*
#  * it uses tabs for indenting (like Guido used to recommend)
#  * it doesn't follow the Maximum Line Length rule


import gedit
import gtk
import os.path
from elastictabstops import to_elastic_tabstops, to_spaces


class ElasticTabstopsConverterPlugin(gedit.Plugin):
	def __init__(self):
		gedit.Plugin.__init__(self)
		self.id_name = 'ElasticTabstopsConverterPluginID'
		self.convert_on_loadsave_enabled = False # change to True to enable convert_on_loadsave by default
		self.tab_size = 0 # this is overwritten

	def replace_docs_contents(self, doc, new_text):
		# replace the doc's contents and try to keep the cursor in roughly the same place
		cursor_iter = doc.get_iter_at_mark(doc.get_insert())
		cursor_line = cursor_iter.get_line()
		cursor_line_offset = cursor_iter.get_line_offset()
		doc.set_text(new_text)
		line_start = doc.get_iter_at_line(cursor_line)
		line_end = line_start.copy()
		line_end.forward_to_line_end()
		line_text = doc.get_text(line_start, line_end)
		if cursor_line_offset < len(line_text):
			new_cursor_iter = doc.get_iter_at_line_offset(cursor_line, cursor_line_offset)
		else:
			new_cursor_iter = line_end
		doc.place_cursor(new_cursor_iter)

	def to_elastic_tabstops_action(self, widget, window):
		doc = window.get_active_document()
		self.tab_size = window.get_active_view().get_tab_width()
		start, end = doc.get_bounds()
		text = doc.get_text(start, end)

		new_text = to_elastic_tabstops(text, self.tab_size)

		doc.begin_user_action() # begins single undo/redo action
		self.replace_docs_contents(doc, new_text)
		doc.end_user_action()

	def to_spaces_action(self, widget, window):
		doc = window.get_active_document()
		self.tab_size = window.get_active_view().get_tab_width()
		start, end = doc.get_bounds()
		text = doc.get_text(start, end)

		new_text = to_spaces(text, self.tab_size)

		doc.begin_user_action() # begins single undo/redo action
		self.replace_docs_contents(doc, new_text)
		doc.end_user_action()

	def convert_on_load_save_action(self, widget, window):
		self.convert_on_loadsave_enabled = (self.convert_on_loadsave_enabled == False)

	def activate(self, window):
		"""Activate plugin."""

		handler_id = window.connect('tab-added', self.on_window_tab_added)
		window.set_data(self.id_name, handler_id)
		self.window = window

		[self.connect_document(doc) for doc in window.get_documents()]

		actions = [
			('toelastictabstops', gtk.STOCK_INDENT,
			'Reindent using tabs (elastic tabstops)', None,
			'Reindent using tabs (elastic tabstops)',
			self.to_elastic_tabstops_action),
			('tospaces', gtk.STOCK_INDENT,
			'Reindent using spaces', None,
			'Reindent using spaces',
			self.to_spaces_action)
		]

		toggleactions = [
			('convertonloadsave', None,
			'Convert to/from elastic tabstops on load/save', None,
			'Convert to/from elastic tabstops on load/save',
			self.convert_on_load_save_action, self.convert_on_loadsave_enabled)
		]

		# store as window data in the window object
		window_data = dict()
		window.set_data('ElasticTabstopsConvertererPluginWindowDataKey', window_data)

		window_data['action_group'] = gtk.ActionGroup('GeditElasticTabstopsConvertererPluginActions')
		#window_data['action_group'].set_translation_domain(GETTEXT_PACKAGE)
		window_data['action_group'].add_actions(actions, window)
		window_data['action_group'].add_toggle_actions(toggleactions, window)

		manager = window.get_ui_manager()
		manager.insert_action_group(window_data['action_group'], -1)
		window_data['ui_id'] = manager.new_merge_id ()

		submenu = '''
		<ui>
			<menubar name="MenuBar">
				<menu name="ToolsMenu" action="Tools">
					<placeholder name="ToolsOps_2">
						<menuitem action="toelastictabstops" />
						<menuitem action="tospaces" />
						<menuitem action="convertonloadsave" />
					</placeholder>
				</menu>
			</menubar>
		</ui>'''

		manager.add_ui_from_string(submenu)

	def connect_document(self, doc):
		"""Connect to document's signals."""

		handler_id = doc.connect('loaded', self.on_document_loaded)
		doc.set_data(self.id_name, [handler_id])

		handler_id = doc.connect('saved', self.on_document_saved)
		doc.set_data(self.id_name, handler_id)

	def deactivate(self, window):
		window_data = window.get_data('ElasticTabstopsConvertererPluginWindowDataKey')
		manager = window.get_ui_manager()
		manager.remove_ui(window_data['ui_id'])
		manager.remove_action_group(window_data['action_group'])

	def on_document_loaded(self, doc, *args):
		"""Called on recieving Loaded signal"""

		if self.convert_on_loadsave_enabled:
			self.tab_size = self.window.get_active_view().get_tab_width()
			start, end = doc.get_bounds()
			text = doc.get_text(start, end)

			new_text = to_elastic_tabstops(text, self.tab_size)

			self.replace_docs_contents(doc, new_text)

	def on_document_saved(self, doc, *args):
		"""Called on recieving Saved signal"""

		if self.convert_on_loadsave_enabled:
			self.tab_size = self.window.get_active_view().get_tab_width()
			start, end = doc.get_bounds()
			text = doc.get_text(start, end)

			new_text = to_spaces(text, self.tab_size)

			filename = os.path.abspath(doc.get_uri_for_display())
			file_handle = open(filename, 'w')
			file_handle.write(new_text)
			file_handle.close()

	def on_window_tab_added(self, window, tab):
		"""Connect the document in tab."""

		self.connect_document(tab.get_document())

	def update_ui(self, window):
		view = window.get_active_view()
		window_data = window.get_data('ElasticTabstopsConvertererPluginWindowDataKey')
		window_data['action_group'].set_sensitive(bool(view and view.get_editable()))
